module structs.endian;

private
{
	import std.bitmanip;
	import std.system;
}

public
{
	/++
	 + Dumps the value with little endian.
	 + Params:
	 +     value =       Input value to dump
	 +     destination = Target output buffer
	 +/
	@nogc pure nothrow void dumpBigEndian(TYPE)(immutable TYPE value, ref ubyte[TYPE.sizeof] destination)
	{
		(cast(ubyte[]) destination).write!(TYPE, Endian.bigEndian)(value, 0);
	}
	/++
	 + Dumps the value in big endian.
	 + Params:
	 +     value =       Input value to dump
	 +     destination = Target output buffer
	 +/
	@nogc pure nothrow void dumpLittleEndian(TYPE)(immutable TYPE value, ref ubyte[TYPE.sizeof] destination)
	{
		(cast(ubyte[]) destination).write!(TYPE, Endian.littleEndian)(value, 0);
	}
	/++
	 + Dumps the value in the system's endian.
	 + Params:
	 +     value =       Input value to dump
	 +     destination = Target output buffer
	 +/
	@nogc pure nothrow void dumpNative(TYPE)(immutable TYPE value, ubyte[TYPE.sizeof] destination)
	{
		static if (endian == Endian.bigEndian)
		{
			dumpBigEndian(value, destination);
		}
		else static if (endian == Endian.littleEndian)
		{
			dumpLittleEndian(value, destination);
		}
		else
		{
			static assert(false);
		}
	}
}

private unittest
{
	// Test dump big endian
	{
		// 1 Byte
		ubyte[1] target = [0];
		dumpBigEndian!byte(1, target);
		assert(target == [1]);

		dumpBigEndian!ubyte(2, target);
		assert(target == [2]);

		dumpBigEndian!char(3, target);
		assert(target == [3]);
	}
	{
		ubyte[2] target = [0, 0];
		dumpBigEndian!short(0x0102, target);
		assert(target == [1, 2]);

		dumpBigEndian!ushort(0x0304, target);
		assert(target == [3, 4]);
	}
	{
		ubyte[4] target = [0, 0, 0, 0];
		dumpBigEndian!int(0x01020304, target);
		assert(target == [1, 2, 3, 4]);

		dumpBigEndian!uint(0x05060708, target);
		assert(target == [5, 6, 7, 8]);
	}
	{
		ubyte[8] target = [0, 0, 0, 0, 0, 0, 0, 0];
		dumpBigEndian!long(0x0102030405060708, target);
		assert(target == [1, 2, 3, 4, 5, 6, 7, 8]);

		dumpBigEndian!ulong(0x090a0b0c0d0e0f10, target);
		assert(target == [9, 10, 11, 12, 13, 14, 15, 16]);
	}

	// Test dump little endian
	{
		// 1 Byte
		ubyte[1] target = [0];
		dumpLittleEndian!byte(1, target);
		assert(target == [1]);

		dumpLittleEndian!ubyte(2, target);
		assert(target == [2]);

		dumpLittleEndian!char(3, target);
		assert(target == [3]);
	}
	{
		ubyte[2] target = [0, 0];
		dumpLittleEndian!short(0x0102, target);
		assert(target == [2, 1]);

		dumpLittleEndian!ushort(0x0304, target);
		assert(target == [4, 3]);
	}
	{
		ubyte[4] target = [0, 0, 0, 0];
		dumpLittleEndian!int(0x01020304, target);
		assert(target == [4, 3, 2, 1]);

		dumpLittleEndian!uint(0x05060708, target);
		assert(target == [8, 7, 6, 5]);
	}
	{
		ubyte[8] target = [0, 0, 0, 0, 0, 0, 0, 0];
		dumpLittleEndian!long(0x0102030405060708, target);
		assert(target == [8, 7, 6, 5, 4, 3, 2, 1]);

		dumpLittleEndian!ulong(0x090a0b0c0d0e0f10, target);
		assert(target == [16, 15, 14, 13, 12, 11, 10, 9]);
	}
}
